@@ -4,12 +4,11 @@ require "rbs/version"
 
 require "set"
 require "json"
-require "pathname" unless defined?(Pathname)
+require "pathname"
 require "pp"
 require "ripper"
 require "logger"
 require "tsort"
-require "strscan"
 
 require "rbs/errors"
 require "rbs/buffer"
